# -----------------------------------------------------------------------------
# >> IMPORTS
# -----------------------------------------------------------------------------

# Python
from random import randint

# Source.Python
from config.manager import ConfigManager
from cvars.flags import ConVarFlags
from cvars.public import PublicConVar
from entities.entity import BaseEntity
from entities.helpers import create_entity
from events import Event
from plugins.info import PluginInfo

# -----------------------------------------------------------------------------
# >> GLOBALS
# -----------------------------------------------------------------------------
EnvEntityDissolver = None

# -----------------------------------------------------------------------------
# >> ADDON INFO
# -----------------------------------------------------------------------------
info = PluginInfo()
info.name = 'Source.Python Dissolver'
info.author = 'your-name-here'
info.version = '1.0'
info.basename = 'dissolver'
info.variable = info.basename + '_version'
info.url = ''
info.convar = PublicConVar(
    info.variable, info.version, 0, info.name + ' Version')

# -----------------------------------------------------------------------------
# >> CONFIG
# -----------------------------------------------------------------------------
with ConfigManager('sp_dissolver') as addon_config:
    addon_config.header = info.name
    
    sp_dissolver_type = addon_config.cvar(
        'sp_dissolver_type',
        '0',
        ConVarFlags.NONE,
        'The type of dissolve effect to use.'
    )
    
    sp_dissolver_magnitude = addon_config.cvar(
        'sp_dissolver_magnitude',
        '2',
        ConVarFlags.NONE,
        'Multiplier of the push force used on the ragdoll while dissolving'
    )
    
    # Describe the available effect types.
    sp_dissolver_type.Options.append('0 => Normal dissolve effect.')
    sp_dissolver_type.Options.append('1 => Electrical effect.')
    sp_dissolver_type.Options.append('2 => Light electrical effect.')
    sp_dissolver_type.Options.append('3 => Core effect.')
    sp_dissolver_type.Options.append('4 => Random effect.')

# -----------------------------------------------------------------------------
# >> EVENTS
# -----------------------------------------------------------------------------
@Event
def round_start( game_event ):
    """
    Called when the round starts. Creates the dissolver entity.
    @param game_event IGameEvent instance containing information
    about the event.
    """
    global EnvEntityDissolver
    
    # Create the dissolver entity. This gets stripped every round
    # so we have to recreate it.
    EnvEntityDissolver = BaseEntity(create_entity('env_entity_dissolver'))
    
    # Setup the dissolve type if it's not random.
    dissolve_type = sp_dissolver_type.get_int()
    
    if dissolve_type != 4:
        EnvEntityDissolver.dissolve_type = dissolve_type

    # Setup magnitude.
    EnvEntityDissolver.magnitude = sp_dissolver_magnitude.get_int()

@Event
def player_death( game_event ):
    """
    Called when a player dies.
    @param game_event IGameEvent instance containing information
    about the event.
    """
    
    # Ensure the entity is valid.
    if EnvEntityDissolver:
        # If we're doing random effects, then pick an effect.
        if sp_dissolver_type.get_int() == 4:
            # Set a random effect.
            EnvEntityDissolver.dissolve_type = randint(0, 3)
    
        # Dissolve the ragdoll entity.
        EnvEntityDissolver.dissolve('cs_ragdoll')
